Package: ggforestplotR
Title: Publication-Ready Forest Plots with 'ggplot2'
Version: 0.3.1
Authors@R: 
    person(given = "Carson",
           family = "Richardson",
           role = c("aut", "cre", "cph"),
           email = "carson.richardson@outlook.com")
Description: Transform model coefficients into flexible forest
    plots using 'ggplot2'. Provides helpers to standardize
    coefficient data from a range of modelling workflows and render
    publication-ready forest plots with a consistent interface.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: ggplot2, patchwork, rlang
Suggests: broom, broom.mixed, dplyr, knitr, lme4, rmarkdown, survival,
        testthat (>= 3.0.0), tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/Needs/website: pkgdown
URL: https://thatoneguy006.github.io/ggforestplotR/,
        https://github.com/thatoneguy006/ggforestplotR
BugReports: https://github.com/thatoneguy006/ggforestplotR/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 20:06:42 UTC; Carso
Author: Carson Richardson [aut, cre, cph]
Maintainer: Carson Richardson <carson.richardson@outlook.com>
Repository: CRAN
Date/Publication: 2026-08-04 22:10:14 UTC
Built: R 4.5.2; ; 2026-08-04 23:13:24 UTC; unix
