# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,ScaleContinuousPosition)
S3method(ggplot_add,ScaleDiscretePosition)
S3method(ggplot_add,ggforestplot_split_table_adder)
S3method(ggplot_add,ggforestplot_table_adder)
export(add_forest_table)
export(add_split_table)
export(as_forest_data)
export(bind_forest_models)
export(ggforestplot)
export(tidy_forest_model)
importFrom(ggplot2,ggplot_add)
importFrom(rlang,.data)
