% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggforestplotR-package.R
\docType{package}
\name{ggforestplotR-package}
\alias{ggforestplotR}
\alias{ggforestplotR-package}
\title{ggforestplotR: Forest plots from model coefficients with ggplot2}
\description{
Transform model coefficients into flexible forest plots using
\code{ggplot2}. Provides helpers to standardize coefficient data
from a range of modelling workflows and render publication-ready
coefficient plots with a consistent interface.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://thatoneguy006.github.io/ggforestplotR/}
  \item \url{https://github.com/thatoneguy006/ggforestplotR}
  \item Report bugs at \url{https://github.com/thatoneguy006/ggforestplotR/issues}
}

}
\author{
\strong{Maintainer}: Carson Richardson \email{carson.richardson@outlook.com} [copyright holder]

}
\keyword{internal}
